@@ -50,8 +50,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'CONFINCLUDEDIR': '/usr/include',
  'CONFINCLUDEPY': '/usr/include/python3.4m',
  'COREPYTHONPATH': ':plat-x86_64-linux-gnu',
- 'COVERAGE_INFO': '/build/buildd/python3.4-3.4.3/build-shared/coverage.info',
- 'COVERAGE_REPORT': '/build/buildd/python3.4-3.4.3/build-shared/lcov-report',
+ 'COVERAGE_INFO': '/build/python3.4-wer9gB/python3.4-3.4.3/build-shared/coverage.info',
+ 'COVERAGE_REPORT': '/build/python3.4-wer9gB/python3.4-3.4.3/build-shared/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov '
                             'report"',
  'CPPFLAGS': '-I. -IInclude -I../Include -D_FORTIFY_SOURCE=2',
@@ -77,7 +77,6 @@ build_time_vars = {'ABIFLAGS': 'm',
  'EXE': '',
  'EXEMODE': 755,
  'EXTRAMACHDEPPATH': '',
- 'EXTRAPLATDIR': '@EXTRAPLATDIR@',
  'EXTRATESTOPTS': '',
  'EXT_SUFFIX': '.cpython-34m.so',
  'FILEMODE': 644,
@@ -502,7 +501,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'LOG1P_DROPS_ZERO_SIGN': 0,
  'MACHDEP': 'linux',
  'MACHDEPPATH': ':plat-x86_64-linux-gnu',
- 'MACHDEPS': 'plat-x86_64-linux-gnu @EXTRAPLATDIR@',
+ 'MACHDEPS': 'plat-x86_64-linux-gnu',
  'MACHDEP_OBJS': '',
  'MACHDESTLIB': '/usr/lib/python3.4',
  'MACOSX_DEPLOYMENT_TARGET': '',
@@ -601,7 +600,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/build/buildd/python3.4-3.4.3/build-shared',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/build/python3.4-wer9gB/python3.4-3.4.3/build-shared',
  'SCRIPTDIR': '/usr/lib',
  'SETPGRP_HAVE_ARG': 0,
  'SGI_ABI': '',
@@ -643,10 +642,10 @@ build_time_vars = {'ABIFLAGS': 'm',
  'TCLTK_LIBS': '',
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/buildd/python3.4-3.4.3/build-shared '
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/python3.4-wer9gB/python3.4-3.4.3/build-shared '
                './python',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/buildd/python3.4-3.4.3/build-shared '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/python3.4-wer9gB/python3.4-3.4.3/build-shared '
                './python ../Tools/scripts/run_tests.py',
  'TESTTIMEOUT': 3600,
  'THREADOBJ': 'Python/thread.o',
@@ -672,8 +671,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/build/buildd/python3.4-3.4.3/build-shared',
- 'abs_srcdir': '/build/buildd/python3.4-3.4.3/build-shared/..',
+ 'abs_builddir': '/build/python3.4-wer9gB/python3.4-3.4.3/build-shared',
+ 'abs_srcdir': '/build/python3.4-wer9gB/python3.4-3.4.3/build-shared/..',
  'datarootdir': '/usr/share',
  'exec_prefix': '/usr',
  'prefix': '/usr',
