@@ -21,7 +21,7 @@
 	,"DEBPKG:debian/deprecate-with-apt - http://bugs.debian.org/747628 Point users to Debian packages of deprecated core modules"
 	,"DEBPKG:debian/squelch-locale-warnings - http://bugs.debian.org/508764 Squelch locale warnings in Debian package maintainer scripts"
 	,"DEBPKG:debian/skip-upstream-git-tests - Skip tests specific to the upstream Git repository"
-	,"DEBPKG:debian/patchlevel - http://bugs.debian.org/567489 List packaged patches for 5.20.2-2ubuntu0.1 in patchlevel.h"
+	,"DEBPKG:debian/patchlevel - http://bugs.debian.org/567489 List packaged patches for 5.20.2-6ubuntu0.2 in patchlevel.h"
 	,"DEBPKG:debian/skip-kfreebsd-crash - http://bugs.debian.org/628493 [perl #96272] Skip a crashing test case in t/op/threads.t on GNU/kFreeBSD"
 	,"DEBPKG:fixes/document_makemaker_ccflags - http://bugs.debian.org/628522 [rt.cpan.org #68613] Document that CCFLAGS should include $Config{ccflags}"
 	,"DEBPKG:debian/find_html2text - http://bugs.debian.org/640479 Configure CPAN::Distribution with correct name of html2text"
@@ -37,4 +37,17 @@
 	,"DEBPKG:fixes/socket_test_recv_fix - http://bugs.debian.org/758718 [perl #122657] Compare recv return value to peername in socket test"
 	,"DEBPKG:fixes/hurd_socket_recv_todo - http://bugs.debian.org/758718 [perl #122657] TODO checking the result of recv() on hurd"
 	,"DEBPKG:fixes/regexp-performance - [0fa70a0] http://bugs.debian.org/777556 [perl #123743] simpify and speed up /.*.../ handling"
+	,"DEBPKG:fixes/failed_require_diagnostics - http://bugs.debian.org/781120 [perl #123270] Report inaccesible file on failed require"
+	,"DEBPKG:fixes/array-cloning - http://bugs.debian.org/779357 [perl #124127] [902d169] fix cloning arrays with unused elements"
+	,"DEBPKG:fixes/perldb-threads - http://bugs.debian.org/779357 [perl #124127] [41ef2c6] lib/perl5db.pl: Restore noop lock prototype"
+	,"DEBPKG:debian/locale-robustness - http://bugs.debian.org/782068 [perl #124310] Make t/run/locale.t survive missing locales masked by LC_ALL"
+	,"DEBPKG:fixes/gcc5-errno - http://bugs.debian.org/778060 [perl #123784] [816b056] Fix Errno.pm generation for gcc-5.0"
+	,"DEBPKG:fixes/h2ph-hex-constants - http://bugs.debian.org/778060 [perl #123784] [3bea78d] h2ph: correct handling of hex constants for the preamble"
+	,"DEBPKG:fixes/h2ph-test-inc - http://bugs.debian.org/778060 [perl #123784] [3359391] lib/h2ph.t to test generated t/_h2ph_pre.ph instead of the system one"
+	,"DEBPKG:fixes/podman-utc - http://bugs.debian.org/780259 Make the embedded date from Pod::Man reproducible"
+	,"DEBPKG:fixes/podman-utc-docs - http://bugs.debian.org/780259 Documentation and test suite updates for UTC fix"
+	,"DEBPKG:fixes/podman-empty-date - http://bugs.debian.org/780259 Support an empty POD_MAN_DATE environment variable"
+	,"DEBPKG:fixes/podman-pipe - http://bugs.debian.org/777405 Better errors for man pages from standard input"
+	,"DEBPKG:debian/pod2man-customized - Update porting/customized.dat for pod2man modifications"
 	,"DEBPKG:fixes/CVE-2015-8607_file_spec_taint_fix - ensure File::Spec::canonpath() preserves taint"
+	,"DEBPKG:fixes/CVE-2016-2381.patch - [PATCH 1/2] remove duplicate environment variables from environ"
