@@ -95,9 +95,9 @@ tie %Config, 'Config', {
     inc_version_list => '5.20.1 5.20.0',
     intsize => '4',
     ldlibpthname => 'LD_LIBRARY_PATH',
-    libpth => '/usr/local/lib /usr/lib/gcc/x86_64-linux-gnu/4.9/include-fixed /usr/include/x86_64-linux-gnu /usr/lib /lib/x86_64-linux-gnu /lib/../lib /usr/lib/x86_64-linux-gnu /usr/lib/../lib /lib',
+    libpth => '/usr/local/lib /usr/lib/gcc/x86_64-linux-gnu/5/include-fixed /usr/include/x86_64-linux-gnu /usr/lib /lib/x86_64-linux-gnu /lib/../lib /usr/lib/x86_64-linux-gnu /usr/lib/../lib /lib',
     osname => 'linux',
-    osvers => '3.13.0-76-generic',
+    osvers => '3.13.0-79-generic',
     path_sep => ':',
     privlibexp => '/usr/share/perl/5.20',
     scriptdir => '/usr/bin',
